"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioInput = void 0;
const stream_1 = require("stream");
const os_1 = require("os");
const ChangeVolume_1 = require("../Utils/ChangeVolume");
const ChangeSampleOptions_1 = require("../Utils/ChangeSampleOptions");
const GenerateSilentChunk_1 = require("../Utils/GenerateSilentChunk");
class AudioInput extends stream_1.Writable {
    inputOptions = {
        sampleRate: 48000,
        channels: 1,
        volume: 100,
        bitDepth: 16,
        endianness: (0, os_1.endianness)(),
        fillChunk: false,
        forceClose: false
    };
    mixerArgs = {
        sampleRate: 48000,
        channels: 1,
        volume: 100,
        bitDepth: 16,
        endianness: (0, os_1.endianness)(),
        highWaterMark: null,
        delayTime: 1,
        autoClose: false
    };
    audioBuffer = Buffer.alloc(0);
    inputClosed = false;
    removeInterval;
    removeSelf;
    constructor(inputArgs, mixerArgs, removeFunction) {
        super();
        this.inputOptions = Object.assign(this.inputOptions, inputArgs);
        this.mixerArgs = Object.assign(this.mixerArgs, mixerArgs);
        this.removeSelf = removeFunction ?? null;
        this.once("close", () => {
            this.inputClosed = true;
            if (this.audioBuffer.length === 0 || this.inputOptions.forceClose) {
                this.audioBuffer = Buffer.alloc(0);
                if (this.removeSelf)
                    this.removeSelf(this);
            }
            else if (this.removeSelf)
                this.removeInterval = setInterval(this.autoRemoveFunc.bind(this), 1);
        });
    }
    _write(chunk, _, callback) {
        if (this.inputClosed)
            return;
        const processedChunk = (0, ChangeSampleOptions_1.default)(chunk, this.inputOptions, this.mixerArgs);
        this.audioBuffer = Buffer.concat([this.audioBuffer, processedChunk]);
        callback();
    }
    get getOptions() {
        return { ...this.inputOptions };
    }
    setVolume(volume) {
        this.inputOptions.volume = volume;
        return this;
    }
    setForceClose(forceClose) {
        this.inputOptions.forceClose = forceClose;
        return this;
    }
    close() {
        this.inputClosed = true;
        if (this.audioBuffer.length === 0 || this.inputOptions.forceClose) {
            this.audioBuffer = Buffer.alloc(0);
            if (this.removeSelf)
                this.removeSelf(this);
        }
        else if (this.removeSelf)
            this.removeInterval = setInterval(this.autoRemoveFunc.bind(this), 1);
        this.end();
    }
    readAudioChunk(highWaterMark) {
        if (this.audioBuffer.length === 0)
            return this.audioBuffer;
        let chunk = this.audioBuffer.subarray(0, highWaterMark ?? this.audioBuffer.length);
        this.audioBuffer = !highWaterMark ? Buffer.alloc(0) : this.audioBuffer.subarray(highWaterMark, this.audioBuffer.length);
        if (typeof highWaterMark === "number") {
            if (chunk.length < highWaterMark) {
                if (!this.inputOptions.fillChunk)
                    return Buffer.alloc(0);
                const silentChunk = (0, GenerateSilentChunk_1.default)(this.mixerArgs.sampleRate, this.mixerArgs.channels, highWaterMark - chunk.length);
                chunk = Buffer.concat([chunk, silentChunk]);
            }
        }
        const changeVolumeArgs = {
            volume: this.inputOptions.volume,
            bitDepth: this.mixerArgs.bitDepth,
            endianness: this.mixerArgs.endianness
        };
        return (0, ChangeVolume_1.default)(chunk, changeVolumeArgs);
    }
    autoRemoveFunc() {
        if (this.audioBuffer.length > 0)
            return;
        clearInterval(this.removeInterval);
        return this.removeSelf(this);
    }
}
exports.AudioInput = AudioInput;
